@@ -25,6 +25,7 @@
 #include <sys/stat.h>
 #include <cerrno>
 #include <fstream>
+#include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <stdexcept>
@@ -32,6 +33,8 @@
 #include <memory>
 #include <unistd.h>
 #include <algorithm>
+#include <regex>
+#include <array>
 
 #ifdef _WIN32
 #define SEPARATOR "\\"
@@ -40,13 +43,6 @@
 #endif
 
 
-unsigned int getFileSize(const std::string& path)
-{
-  struct stat filestatus;
-  stat(path.c_str(), &filestatus);
-  return filestatus.st_size;
-}
-
 bool fileExists(const std::string& path)
 {
   bool flag = false;
@@ -159,32 +155,63 @@ static std::string removeLastPathElement(const std::string& path,
   return newPath;
 }
 
-/* Split string in a token array */
-static std::vector<std::string> split(const std::string& str,
-                               const std::string& delims = " *-")
+namespace
 {
-  std::string::size_type lastPos = str.find_first_not_of(delims, 0);
-  std::string::size_type pos = str.find_first_of(delims, lastPos);
-  std::vector<std::string> tokens;
 
-  while (std::string::npos != pos || std::string::npos != lastPos) {
-    tokens.push_back(str.substr(lastPos, pos - lastPos));
-    lastPos = str.find_first_not_of(delims, pos);
-    pos = str.find_first_of(delims, lastPos);
+std::vector<std::string> split(const std::string & str, char delim)
+{
+  std::vector<std::string> result;
+  auto start = str.begin();
+  while ( true ) {
+      const auto end = find(start, str.end(), delim);
+
+      result.push_back(std::string(start, end));
+      if ( end == str.end() )
+        break;
+      start = end + 1;
   }
 
-  return tokens;
+  return result;
 }
 
-static std::vector<std::string> split(const char* lhs, const char* rhs)
+std::string
+getRelativePath(const std::string& basePath, const std::string& targetPath)
 {
-  const std::string m1(lhs), m2(rhs);
-  return split(m1, m2);
+    const auto b = split(basePath, '/');
+    const auto t = split(targetPath, '/');
+    const auto l = std::min(b.size()-1, t.size());
+
+    auto x = mismatch(b.begin(), b.begin() + l, t.begin());
+
+    const size_t ups = (b.end() - x.first) + (x.second == t.end());
+    std::string r;
+    for ( size_t i = 1; i < ups; ++i ) {
+      if ( !r.empty() && r.back() != '/' )
+        r += "/";
+      r += "..";
+    }
+
+    for ( auto it = x.second; it != t.end(); ++it ) {
+        if ( !r.empty() && r.back() != '/' )
+          r += "/";
+        r += *it;
+    }
+    return r;
 }
 
-static std::vector<std::string> split(const std::string& lhs, const char* rhs)
+} // unnamed namespace
+
+std::string
+computeRelativePath(const std::string& basePath, const std::string& targetPath)
 {
-  return split(lhs.c_str(), rhs);
+  if ( targetPath.back() == '/' ) {
+    if ( basePath == targetPath )
+      return "./";
+
+    const std::string strippedPath = targetPath.substr(0, targetPath.size()-1);
+    return getRelativePath(basePath, strippedPath) + '/';
+  }
+  return getRelativePath(basePath, targetPath);
 }
 
 /* Warning: the relative path must be with slashes */
@@ -192,9 +219,11 @@ std::string computeAbsolutePath(const std::string& path,
                                 const std::string& relativePath)
 {
   /* Remove leaf part of the path if not already a directory */
-  std::string absolutePath = path[path.length() - 1] == '/'
-                                 ? path
-                                 : removeLastPathElement(path, false, false);
+  std::string absolutePath = path.length()
+                               ? (path[path.length() - 1] == '/'
+                                  ? path
+                                  : removeLastPathElement(path, false, false))
+                               : path;
 
   /* Go through relative path */
   std::vector<std::string> relativePathElements;
@@ -213,38 +242,6 @@ std::string computeAbsolutePath(const std::string& path,
   return absolutePath.substr(0, absolutePath.length() - 1);
 }
 
-/* Warning: the relative path must be with slashes */
-std::string computeRelativePath(const std::string path,
-                                const std::string absolutePath)
-{
-  if (path.empty())
-    return "";
-
-  std::vector<std::string> pathParts = split(path, "/");
-  std::vector<std::string> absolutePathParts = split(absolutePath, "/");
-
-  unsigned int commonCount = 0;
-  while (commonCount < pathParts.size()
-         && commonCount < absolutePathParts.size()
-         && pathParts[commonCount] == absolutePathParts[commonCount]) {
-    if (!pathParts[commonCount].empty()) {
-      commonCount++;
-    }
-  }
-
-  std::string relativePath;
-  for (unsigned int i = commonCount; i < pathParts.size() - 1; i++) {
-    relativePath += "../";
-  }
-
-  for (unsigned int i = commonCount; i < absolutePathParts.size(); i++) {
-    relativePath += absolutePathParts[i];
-    relativePath += i + 1 < absolutePathParts.size() ? "/" : "";
-  }
-
-  return relativePath;
-}
-
 
 void replaceStringInPlaceOnce(std::string& subject,
                               const std::string& search,
@@ -279,53 +276,6 @@ void stripTitleInvalidChars(std::string& str)
   replaceStringInPlace(str, "\u202C", "");
 }
 
-std::string getNamespaceForMimeType(const std::string& mimeType, bool uniqueNamespace)
-{
-  if (uniqueNamespace || mimeType.find("text") == 0 || mimeType.empty()) {
-    if (uniqueNamespace || mimeType.find("text/html") == 0
-        || mimeType.empty()) {
-      return "A";
-    } else {
-      return "-";
-    }
-  } else {
-    if (mimeType == "application/font-ttf"
-        || mimeType == "application/font-woff"
-        || mimeType == "application/font-woff2"
-        || mimeType == "application/vnd.ms-opentype"
-        || mimeType == "application/vnd.ms-fontobject"
-        || mimeType == "application/javascript"
-        || mimeType == "application/json") {
-      return "-";
-    } else {
-      return "I";
-    }
-  }
-}
-
-void remove_all(const std::string& path)
-{
-  DIR* dir;
-
-  /* It's a directory, remove all its entries first */
-  if ((dir = opendir(path.c_str())) != NULL) {
-    struct dirent* ent;
-    while ((ent = readdir(dir)) != NULL) {
-      if (strcmp(ent->d_name, ".") and strcmp(ent->d_name, "..")) {
-        std::string childPath = path + SEPARATOR + ent->d_name;
-        remove_all(childPath);
-      }
-    }
-    closedir(dir);
-    rmdir(path.c_str());
-  }
-
-  /* It's a file */
-  else {
-    remove(path.c_str());
-  }
-}
-
 std::vector<html_link> generic_getLinks(const std::string& page)
 {
     const char* p = page.c_str();
@@ -359,7 +309,8 @@ std::vector<html_link> generic_getLinks(const std::string& page)
         // [TODO] Handle escape char
         while(*p != delimiter)
             p++;
-        links.push_back({attr,std::string(linkStart, p)});
+        const std::string link(linkStart, p);
+        links.push_back(html_link(attr, link));
         p += 1;
     }
     return links;
@@ -369,7 +320,7 @@ bool isOutofBounds(const std::string& input, std::string base)
 {
     if (input.empty()) return false;
 
-    if (base.back() != '/')
+    if (!base.length() || base.back() != '/')
         base.push_back('/');
 
     int nr = 0;
@@ -414,7 +365,7 @@ std::string normalize_link(const std::string& input, const std::string& baseUrl)
     } else {
       //This is a relative url, use base url
       output = baseUrl;
-      if (output.back() != '/')
+      if (!output.empty() && output.back() != '/')
           output += '/';
       check_rel = true;
     }
@@ -470,3 +421,114 @@ std::string normalize_link(const std::string& input, const std::string& baseUrl)
     }
     return output;
 }
+
+namespace
+{
+
+UriKind specialUriSchemeKind(const std::string& s)
+{
+    static const std::map<std::string, UriKind> uriSchemes = {
+        { "javascript", UriKind::JAVASCRIPT },
+        { "mailto",     UriKind::MAILTO     },
+        { "tel",        UriKind::TEL        },
+        { "geo",        UriKind::GEO        },
+        { "data",       UriKind::DATA       },
+        { "xmpp",       UriKind::XMPP       },
+        { "news",       UriKind::NEWS       },
+        { "urn",        UriKind::URN        }
+    };
+
+    const auto it = uriSchemes.find(s);
+    return it != uriSchemes.end() ? it->second : UriKind::OTHER;
+}
+
+void asciitolower(std::string& s)
+{
+    std::transform(s.begin(), s.end(), s.begin(), [](unsigned char c){
+        return ('A' <= c && c <= 'Z') ? c - ('Z' - 'z') : c;
+    });
+}
+
+} // unnamed namespace
+
+UriKind html_link::detectUriKind(const std::string& input_string)
+{
+    const auto k = input_string.find_first_of(":/?#");
+    if ( k == std::string::npos || input_string[k] != ':' )
+        return UriKind::OTHER;
+
+    if ( k + 2 < input_string.size()
+         && input_string[k+1] == '/'
+         && input_string[k+2] == '/' )
+        return UriKind::GENERIC_URI;
+
+    std::string scheme = input_string.substr(0, k);
+    asciitolower(scheme);
+    return specialUriSchemeKind(scheme);
+}
+
+namespace
+{
+
+static bool isReservedUrlChar(const char c)
+{
+    constexpr std::array<char, 10> reserved = {{';', ',', '?', ':',
+                                               '@', '&', '=', '+', '$' }};
+
+    return std::any_of(reserved.begin(), reserved.end(),
+                       [&c] (const char &elem) { return elem == c; } );
+}
+
+bool needsEscape(const char c, const bool encodeReserved)
+{
+  if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || (c >= '0' && c <= '9'))
+    return false;
+
+  if (isReservedUrlChar(c))
+    return encodeReserved;
+
+  constexpr std::array<char, 10> noNeedEscape = {{'-', '_', '.', '!', '~',
+                                                '*', '\'', '(', ')', '/' }};
+
+  return not std::any_of(noNeedEscape.begin(), noNeedEscape.end(),
+                         [&c] (const char &elem) { return elem == c; } );
+}
+
+std::string urlEncode(const std::string& value, bool encodeReserved)
+{
+  std::ostringstream os;
+  os << std::hex << std::uppercase;
+  for (std::string::const_iterator it = value.begin();
+       it != value.end();
+       ++it) {
+    if (!needsEscape(*it, encodeReserved)) {
+      os << *it;
+    } else {
+      os << '%' << std::setw(2) << static_cast<unsigned int>(static_cast<unsigned char>(*it));
+    }
+  }
+  return os.str();
+}
+
+} // unnamed namespace
+
+std::string httpRedirectHtml(const std::string& redirectUrl)
+{
+    const auto encodedurl = urlEncode(redirectUrl, true);
+    std::ostringstream ss;
+
+    ss << "<!DOCTYPE html>"
+          "<html>"
+          "<head>"
+          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
+          "<meta http-equiv=\"refresh\" content=\"0;url=" + encodedurl + "\" />"
+          "</head>"
+          "<body></body>"
+          "</html>";
+    return ss.str();
+}
+
+bool guess_is_front_article(const std::string& mimetype) {
+  return ( mimetype.find("text/html") == 0
+        && mimetype.find("raw=true") == std::string::npos);
+}
