@@ -18,9 +18,11 @@
  */
 
 #include <fstream>
+#include <sstream>
 #include <stdexcept>
 
-#include <zim/file.h>
+#define ZIM_PRIVATE
+#include <zim/archive.h>
 
 #include <docopt/docopt.h>
 
@@ -33,7 +35,7 @@
 class ZimSplitter
 {
   private:
-    zim::File file;
+    zim::Archive archive;
     const std::string prefix;
     zim::size_type partSize;
 
@@ -47,7 +49,7 @@ class ZimSplitter
 
   public:
     ZimSplitter(const std::string& fname, const std::string& out_prefix, zim::size_type partSize)
-      : file(fname),
+      : archive(fname),
         prefix(out_prefix),
         partSize(partSize),
         first_index(0),
@@ -93,10 +95,6 @@ class ZimSplitter
         out_size = 0;
     }
 
-    void write_first_part() {
-       copy_out(file.getClusterOffset(0));
-    }
-
     void copy_out(zim::size_type size) {
         while (size > 0) {
            auto size_to_copy = std::min<zim::size_type>(size, BUFFER_SIZE);
@@ -113,51 +111,61 @@ class ZimSplitter
         }
     }
 
+    std::vector<zim::offset_type> getOffsets() {
+        std::vector<zim::offset_type> offsets;
+        auto nbClusters = archive.getClusterCount();
+        for(zim::offset_type i=0;i<nbClusters;i++) {
+            offsets.push_back(archive.getClusterOffset(i));
+        }
+        offsets.push_back(archive.getFilesize());
+        std::sort(offsets.begin(), offsets.end());
+        return offsets;
+    }
+
     void run() {
-       new_file();
-       write_first_part();
+        new_file();
 
-       auto nbClusters = file.getFileheader().getClusterCount();
-       for(unsigned int cluster_nb=0;cluster_nb<nbClusters-1; cluster_nb++) {
-           auto clusterOffset = file.getClusterOffset(cluster_nb);
-           auto clusterEnd = file.getClusterOffset(cluster_nb+1);
-           auto clusterSize = clusterEnd-clusterOffset;
-           if ( out_size+clusterSize > partSize ) {
-               new_file();
-           }
-           copy_out(clusterSize);
-       }
+        auto offsets = getOffsets();
 
-       auto lastClusterOffset = file.getClusterOffset(nbClusters-1);
-       auto lastPartSize = file.getFilesize() - lastClusterOffset;
-
-       if (out_size+lastPartSize > partSize) {
-           new_file();
-       }
-       copy_out(lastPartSize);
+        zim::offset_type last(0);
+        for(auto offset:offsets) {
+            auto currentSize = offset-last;
+            if (currentSize > partSize) {
+                // One part is bigger than what we want :/
+                // Still have to write it.
+                if (out_size) {
+                    new_file();
+                }
+                copy_out(currentSize);
+                new_file();
+            } else {
+                if (out_size+currentSize > partSize) {
+                    // It would be too much to write the current part in the current file.
+                    new_file();
+                }
+                copy_out(currentSize);
+            }
+            last = offset;
+        }
     }
 
     bool check() {
-       bool error = false;
-       auto nbClusters = file.getFileheader().getClusterCount();
-       for(unsigned int cluster_nb=0;cluster_nb<nbClusters-1; cluster_nb++) {
-           auto clusterOffset = file.getClusterOffset(cluster_nb);
-           auto clusterEnd = file.getClusterOffset(cluster_nb+1);
-           auto clusterSize = clusterEnd-clusterOffset;
-           if (clusterSize > partSize) {
-               // The cluster size is to big, even to fit in a new file part.
-               std::cout << "Cluster " << cluster_nb << " is to big to fit in one part." << std::endl;
-               error = true;
-           }
-       }
+        bool error = false;
 
-       auto lastClusterOffset = file.getClusterOffset(nbClusters-1);
-       auto lastPartSize = file.getFilesize() - lastClusterOffset;
+        auto offsets = getOffsets();
 
-       if (lastPartSize > partSize) {
-           std::cout << "Last cluster and checksum are too big to fit in one part." << std::endl;
-           error = true;
-       }
+        zim::offset_type last(0);
+        for(auto offset:offsets) {
+            auto currentSize = offset-last;
+            if (currentSize > partSize) {
+                // One part is bigger than what we want :/
+                // Still have to write it.
+                std::cout << "The part (probably a cluster) is to big to fit in one part." << std::endl;
+                std::cout << "    size is " << currentSize << "(" << offset << "-" << last << ")." << std::endl;
+                error = true;
+            }
+            last = offset;
+        }
        return error;
     }
 };
@@ -181,12 +189,12 @@ int main(int argc, char* argv[])
 {
   try
   {
-
-    std::string versionstr("zimsplit " + std::string(VERSION));
+    std::ostringstream versions;
+    printVersions(versions);
     std::map<std::string, docopt::value> args = docopt::docopt(USAGE,
-                                                            {argv + 1, argv + argc},
-                                                            true,
-                                                            versionstr);
+                                                               {argv + 1, argv + argc},
+                                                               true,
+                                                               versions.str());
 
     std::string prefix = args["<file>"].asString();
     if (args["--prefix"])
